% Write STAG_3D format from matlab in XML-VTK format, using ASCII or BINARY file
% format
%
%
%
% Boris Kaus, 21 Feb. 2008


ASCII           = logical(0);                       % ASCII or BINARY?

% Change to correct directory
start_dir       = pwd;
% cd(directory);
cd(saveDirectory);


% Generate input data in correct format
Points          = single([X_3D(:) Y_3D(:) Z_3D(:)]);
T_3D            = single(T_3D);


% Transform the arrays in vector-format, and change them to the appropriate
% precision. This example file assumes that data comes in single precision
%
% Double precision requires changing in ths code below (Float32 into Float64)
T               = single(T_3D(:));                                                  % Temperature
Points          = [single(X_3D(:))  single(Y_3D(:))     single(Z_3D(:))     ];      % Coordinates

% Define which arrays to write
if WriteResidualT
   Tres             = single(Tres_3D(:));                                                  % Temperature
end

if WriteVelocity
    P               = single(P_3D(:));                                                  % Pressure
    Velocity        = [single(VX_3D(:)) single(VY_3D(:))    single(VZ_3D(:))    ];      % Velocity                                       % Velocity @ every point
end

if WriteViscosity
    Eta             = single(ETA_3D(:));                                                % Viscosity
end

if WriteStress
    Stress     = single(C_3D(:));                                                % Stress
end

if WriteStrainrate
    Edot     = single(C_3D(:));                                                % Strainrate
end

if WriteComposition
    Composition     = single(C_3D(:));                                                % Composition
end

if WriteMeltFrac
    MeltFraction     = single(F_3D(:));                                                % Melt Fraction
end

if WriteDefMech
    Defmech     = single(C_3D(:));                                                % Defmech
end

%==========================================================================
% Definitions and initialization
sizeof_Float32  =   4;      
sizeof_Float64  =   4;     
sizeof_UInt32   =   4;     
Offset          =   0;      % Initial offset

%--------------------------------------------------------------------------
% Write the header for a structured grid:
%--------------------------------------------------------------------------
fname_vtk       = [fname,'_',num2str(1000000+fname_number),'.vts'];
fid             = fopen(fname_vtk,'w','b');           % note the 'b': not doing BigEndian does not work with MATLAB!
fprintf(fid,'<?xml version="1.0"?> \n');
fprintf(fid,'<VTKFile type="StructuredGrid" version="0.1" byte_order="BigEndian" >\n');
fprintf(fid,'  <StructuredGrid  WholeExtent="%i %i %i %i %i %i">\n', [0 size(X_3D,1)-1 0 size(X_3D,2)-1 0 size(X_3D,3)-1]);
fprintf(fid,'  <Piece Extent="%i %i %i %i %i %i">\n', [0 size(X_3D,1)-1 0 size(X_3D,2)-1 0 size(X_3D,3)-1]);

%--------------------------------------------------------------------------
% Add point-wise data
%--------------------------------------------------------------------------
fprintf(fid,'    <PointData Scalars="T" Vectors="Velocity"  >\n');

% TEMPERATURE -----------
if ASCII
    % ASCII:
    fprintf(fid,'      <DataArray type="Float32" Name="T" format="ascii">\n');
    for i=1:length(T)
        fprintf(fid,'        %g \n',single(T(i)));
    end
else
    % BINARY:
    fprintf(fid,'      <DataArray type="Float32" Name="T" format="appended" offset="%i">\n', int32(Offset));
    Offset = Offset + length(T(:))*sizeof_Float32 + 1*sizeof_UInt32;
end
fprintf(fid,'      </DataArray>\n');
% -----------------------

if WriteResidualT
    % RESIDUAL TEMPERATURE -----
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="Tres" format="ascii">\n');
        for i=1:length(Tres)
            fprintf(fid,'        %g \n',single(Tres(i)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="Tres" format="appended" offset="%i">\n', int32(Offset));
        Offset = Offset + length(Tres(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------
end

if WriteViscosity
    % VISCOSITY---------------
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="Viscosity" format="ascii">\n');
        for i=1:length(Eta)
            fprintf(fid,'        %g \n',single(Eta(i)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="Viscosity" format="appended" offset="%i">\n',int32(Offset));
        Offset = Offset + length(Eta(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------
end

if WriteStress
    % STRESS---------------
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="Stress" format="ascii">\n');
        for i=1:length(Stress)
            fprintf(fid,'        %g \n',single(Stress(i)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="Stress" format="appended" offset="%i">\n',int32(Offset));
        Offset = Offset + length(Stress(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------
end

if WriteStrainrate
    % STRAINRATE---------------
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="Strainrate" format="ascii">\n');
        for i=1:length(Edot)
            fprintf(fid,'        %g \n',single(Edot(i)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="Strainrate" format="appended" offset="%i">\n',int32(Offset));
        Offset = Offset + length(Edot(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------
end

if WriteComposition
    % COMPOSITION---------------
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="Composition" format="ascii">\n');
        for i=1:length(Composition)
            fprintf(fid,'        %g \n',single(Composition(i)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="Composition" format="appended" offset="%i">\n',int32(Offset));
        Offset = Offset + length(Composition(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------
end


if WriteMeltFrac
    % MELT FRACTION---------------
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="Melt Fraction" format="ascii">\n');
        for i=1:length(MeltFraction)
            fprintf(fid,'        %g \n',single(MeltFraction(i)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="Melt Fraction" format="appended" offset="%i">\n',int32(Offset));
        Offset = Offset + length(MeltFraction(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------
end

if WriteDefMech
    % DefMech---------------
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="Composition" format="ascii">\n');
        for i=1:length(DefMech)
            fprintf(fid,'        %g \n',single(DefMech(i)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="Composition" format="appended" offset="%i">\n',int32(Offset));
        Offset = Offset + length(DefMech(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------
end

if WriteVelocity
    
    % PRESSURE---------------
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="P" format="ascii">\n');
        for i=1:length(P)
            fprintf(fid,'        %g \n',single(P(i)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="P" format="appended" offset="%i">\n',int32(Offset));
        Offset = Offset + length(P(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------

    % VELOCITY---------------  : VELOCITY IS A 3-component vector
    if ASCII
        % ASCII:
        fprintf(fid,'      <DataArray type="Float32" Name="Velocity" NumberOfComponents="3" format="ascii">\n');
        for i=1:length(T)
            fprintf(fid,'   %g %g %g \n',single(Velocity(i,:)));
        end
    else
        % BINARY:
        fprintf(fid,'      <DataArray type="Float32" Name="Velocity" NumberOfComponents="3" format="appended" offset="%i">\n',int32(Offset));
        Offset = Offset + length(Velocity(:))*sizeof_Float32 + 1*sizeof_UInt32;
    end
    fprintf(fid,'      </DataArray>\n');
    % -----------------------
    
end


fprintf(fid,'    </PointData>\n');
%--------------------------------------------------------------------------
% 
%--------------------------------------------------------------------------

fprintf(fid,'    <Celldata>\n');
fprintf(fid,'    </Celldata>\n');


%--------------------------------------------------------------------------
% Add coordinates of structured grid 
%--------------------------------------------------------------------------
fprintf(fid,'    <Points>\n');

% ASCII
if ASCII
    fprintf(fid,'      <DataArray type="Float32" Name="Array" NumberOfComponents="3" format="ascii">\n');
    for i=1:size(Points,1)
        fprintf(fid,' %g %g %g \n',[Points(i,:)]);
    end
else
    fprintf(fid,'      <DataArray type="Float32" Name="Array" NumberOfComponents="3" format="appended" offset="%i" >\n',int32(Offset));
    
end
fprintf(fid,'      </DataArray>\n');
fprintf(fid,'    </Points>\n');
%--------------------------------------------------------------------------

fprintf(fid,'  </Piece> \n');
fprintf(fid,'  </StructuredGrid> \n');


if ~ASCII
    % Append binary data in raw format: the order in which data arrays are
    % added should be the same as how they are defined above
    fprintf(fid,'  <AppendedData encoding="raw"> \n');
    fprintf(fid,'_');

    % Add Temperature in binary format
    fwrite(fid,uint32(length(T)*sizeof_Float32),'uint32');
    fwrite(fid,single(T).'      ,   'float32');
    
    if WriteResidualT
        % Add Viscosity in binary format
        fwrite(fid,uint32(length(Tres)*sizeof_Float32),'uint32');
        fwrite(fid,single(Tres).'      ,   'float32');
    end
    
    if WriteViscosity
        % Add Viscosity in binary format
        fwrite(fid,uint32(length(Eta)*sizeof_Float32),'uint32');
        fwrite(fid,single(Eta).'      ,   'float32');
    end
    
    if WriteStress
        % Add Stress in binary format
        fwrite(fid,uint32(length(Stress)*sizeof_Float32),'uint32');
        fwrite(fid,single(Stress).'      ,   'float32');
    end
    
    if WriteStrainrate
        % Add Strainrate in binary format
        fwrite(fid,uint32(length(Edot)*sizeof_Float32),'uint32');
        fwrite(fid,single(Edot).'      ,   'float32');
    end
    
    if WriteComposition
        % Add Composition in binary format
        fwrite(fid,uint32(length(Composition)*sizeof_Float32),'uint32');
        fwrite(fid,single(Composition).'      ,   'float32');
    end

    if WriteMeltFrac
        % Add Melt fraction in binary format
        fwrite(fid,uint32(length(MeltFraction)*sizeof_Float32),'uint32');
        fwrite(fid,single(MeltFraction).'      ,   'float32');
    end
    
    if WriteDefMech
        % Add Composition in binary format
        fwrite(fid,uint32(length(DefMech)*sizeof_Float32),'uint32');
        fwrite(fid,single(DefMech).'      ,   'float32');
    end

    if WriteVelocity
        % Add Pressure in binary format
        fwrite(fid,uint32(length(P)*sizeof_Float32),'uint32');
        fwrite(fid,single(P).'      ,   'float32');

        % Add Velocity in binary format
        fwrite(fid,uint32(length(Velocity(:))*sizeof_Float32),'uint32');
        fwrite(fid,single(Velocity).' ,   'float32');
    end

    % Add Coordinates in binary format
    fwrite(fid,uint32(length(Points(:))*sizeof_Float32),'uint32');
    fwrite(fid,single(Points).' ,   'float32');

    fprintf(fid,'  </AppendedData> \n');
end


fprintf(fid,'</VTKFile>\n');
fclose(fid);

cd(start_dir)

if ~ASCII
    disp(['   Created Binary XML-VTK output file ',fname_vtk])
else
    disp(['   Created ASCII XML-VTK output file ',fname_vtk])
end